/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.hierarchy;

import org.databene.commons.Filter;
import org.databene.commons.filter.FilterUtil;
import org.databene.dbsanity.HierarchyProvider;
import org.databene.dbsanity.model.SanityCheck;

/**
 * {@link HierarchyProvider} which is configured with a tag namespace and interprets check tags as paths.
 * For example, if the namespace is 'p' (which is the default), a check with a tag 'p:alpha/beta' is 
 * put into a group 'beta' which is made a sub group of 'alpha'.<br/><br/>
 * Created: 13.11.2011 06:03:10
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class TagHierarchyProvider implements HierarchyProvider {
	
	private static final String[] ROOT_PATH = new String[0];
	
	private String namespace;
	private String prefix;
	
	public TagHierarchyProvider() {
		this("p");
	}

	public TagHierarchyProvider(String namespace) {
		setNamespace(namespace);
	}
	
	public String getNamespace() {
		return namespace;
	}

	protected void setNamespace(String namespace) {
		this.namespace = namespace;
		this.prefix = namespace + ":";
	}

	public String getName() {
		return "Tag Hierarchy";
	}

	public String[] getParentPath(SanityCheck check) {
		String pathTag = FilterUtil.findSingleMatch(check.getTags(), new Filter<String>() {
			public boolean accept(String candidate) {
				return candidate.startsWith(prefix);
			}
		});
		if (pathTag == null)
			return ROOT_PATH;
		pathTag = pathTag.substring(prefix.length());
		return pathTag.split("/");
	}

}
