/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

import java.util.Map;

import org.databene.commons.ArrayBuilder;
import org.databene.commons.collection.OrderedNameMap;

/**
 * Provides generic rendering features for HTML elements.<br/><br/>
 * Created: 03.12.2011 09:12:32
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class HtmlElement<E extends HtmlElement<?>> implements Renderable {

	protected String tagName;
	protected boolean inline;
	protected OrderedNameMap<String> attributes;
	
	public HtmlElement(String tagName, boolean inline) {
		this.tagName = tagName;
		this.inline = inline;
		this.attributes = OrderedNameMap.createCaseInsensitiveMap();
	}
	
	public E withClass(String klass) {
		return this.withAttribute("class", klass);
	}

	public E withStyle(String style) {
		return this.withAttribute("style", style);
	}
	
	protected void renderStartTagWithAttributes(HtmlPrintWriter out) {
		ArrayBuilder<String> builder = new ArrayBuilder<String>(String.class);
		for (Map.Entry<String, String> entry : attributes.entrySet()) {
			builder.add(entry.getKey());
			builder.add(entry.getValue());
		}
		out.startTag(tagName, inline, builder.toArray());
	}
	
	protected void renderEndTag(HtmlPrintWriter out) {
		out.endTag(tagName, inline);
	}
	
	@SuppressWarnings("unchecked")
	protected E withAttribute(String attributeName, String attributeValue) {
		attributes.put(attributeName, attributeValue);
		return (E) this;
	}

	public void render(HtmlPrintWriter out) {
		renderStartTagWithAttributes(out);
		renderEndTag(out);
	}

}
