/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

import java.io.File;

/**
 * Abstract representation of an HTML image.<br/><br/>
 * Created: 03.12.2011 07:55:44
 * @since 0.9.3
 * @author Volker Bergmann
 */
public abstract class HtmlImage<E extends HtmlImage<?>> extends HtmlElement<E> {
	
	public HtmlImage() {
		super("img", true);
	}

	public E withTitle(String title) {
		return this.withAttribute("title", title);
	}
	
	public E withWidth(String width) {
		return this.withAttribute("width", width);
	}
	
	public E withHeight(String height) {
		return this.withAttribute("height", height);
	}

	public void render(HtmlPrintWriter out) {
		this.withAttribute("src", renderSourceUrl(out.getFile()));
		super.render(out);
	}
	
	protected abstract String renderSourceUrl(File fromFile);
	
}
