/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

/**
 * Simple HTML element which expects {@link Renderable} content.<br/><br/>
 * Created: 03.12.2011 09:07:03
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class HtmlSimpleContentElement extends HtmlElement<HtmlSimpleContentElement> {
	
	protected Renderable content;
	
	public HtmlSimpleContentElement(String tagName, String content) {
		this(tagName, new RenderableText(content));
	}
	
	public HtmlSimpleContentElement(String tagName) {
		this(tagName, (Renderable) null);
	}
	
	public HtmlSimpleContentElement(String tagName, Renderable content) {
		super(tagName, true);
		this.content = content;
	}
	
	public HtmlSimpleContentElement withContent(String content) {
		return withContent(new RenderableText(content));
	}
	
	public HtmlSimpleContentElement withContent(Renderable content) {
		this.content = content;
		return this;
	}
	
	public void render(HtmlPrintWriter out) {
		renderStartTagWithAttributes(out);
		if (content != null)
			content.render(out);
		renderEndTag(out);
	}
	
}
