/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

import java.util.List;

import org.databene.commons.CollectionUtil;

/**
 * Represents an HTML table.<br/><br/>
 * Created: 04.12.2011 07:58:50
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class HtmlTable extends HtmlElement<HtmlTable> {
	
	private List<HtmlTableRow> rows;

	public HtmlTable(HtmlTableRow... rows) {
		super("table", false);
		withRows(rows);
	}

	public HtmlTable withRows(HtmlTableRow... rows) {
		this.rows = CollectionUtil.toList(rows);
		return this;
	}

	public HtmlTable withWidth(String width) {
		return this.withAttribute("width", width);
	}

	public void render(HtmlPrintWriter out) {
		renderStartTagWithAttributes(out);
		for (HtmlTableRow row : rows)
			row.render(out);
		renderEndTag(out);
	}

}
