/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

/**
 * Represents a table cell.<br/><br/>
 * Created: 02.12.2011 19:00:55
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class HtmlTableCell extends HtmlElement<HtmlTableCell> {

	private Renderable content;
	
	public HtmlTableCell() {
		this((Renderable) null);
	}
	
	public HtmlTableCell(int content) {
		this(new RenderableText(String.valueOf(content)));
	}
	
	public HtmlTableCell(String content) {
		this(new RenderableText(content));
	}
	
	public HtmlTableCell(Renderable content) {
		this(content, "td");
	}
	
	public HtmlTableCell(Renderable content, String tag) {
		super(tag, true);
		this.content = content;
	}
	
	public HtmlTableCell withAlignment(String align) {
		return this.withAttribute("align", align);
	}

	public HtmlTableCell leftAligned() {
		return this.withAlignment("left");
	}

	public HtmlTableCell rightAligned() {
		return this.withAlignment("right");
	}

	public HtmlTableCell topAligned() {
		return this.withAttribute("valign", "top");
	}

	public HtmlTableCell centered() {
		return this.withAlignment("center");
	}

	public HtmlTableCell withColspan(int colspan) {
		return this.withAttribute("colspan", String.valueOf(colspan));
	}
	
	public HtmlTableCell withWidth(String width) {
		return this.withAttribute("width", width);
	}
	
	public HtmlTableCell withContent(String content) {
		return withContent(new RenderableText(content));
	}

	public HtmlTableCell withContent(Renderable content) {
		this.content = content;
		return this;
	}

	public void render(HtmlPrintWriter out) {
		renderStartTagWithAttributes(out);
		if (content != null)
			content.render(out);
		renderEndTag(out);
		out.println();
	}

}
