/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

import java.util.List;

import org.databene.commons.CollectionUtil;

/**
 * Represents an HTML table row.<br/><br/>
 * Created: 02.12.2011 19:12:54
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class HtmlTableRow extends HtmlElement<HtmlTableRow> {
	
	private List<HtmlTableCell> cells;

	public HtmlTableRow(HtmlTableCell... cells) {
		super("tr", false);
		withCells(cells);
	}

	public HtmlTableRow withCells(HtmlTableCell...cells) {
		this.cells = CollectionUtil.toList(cells);
		return this;
	}

	public HtmlTableRow topAligned() {
		this.attributes.put("valign", "top");
		return this;
	}

	public void render(HtmlPrintWriter out) {
		renderStartTagWithAttributes(out);
		for (HtmlTableCell cell : cells)
			cell.render(out);
		renderEndTag(out);
	}

}
