/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.html;

/**
 * Represents a reference to a URL.<br/><br/>
 * Created: 04.12.2011 08:45:53
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class HtmlUrlReference extends HtmlElement<HtmlUrlReference> {
	
	private Renderable label;
	
	public HtmlUrlReference(String label, String url) {
		this(new RenderableText(label), url);
	}
	
	public HtmlUrlReference(Renderable label, String url) {
		super("a", true);
		this.label = label;
		withUrl(url);
	}
	
	public HtmlUrlReference withUrl(String url) {
		return this.withAttribute("href", url);
	}

	public HtmlUrlReference withTarget(String target) {
		return this.withAttribute("target", target);
	}
	
	public void render(HtmlPrintWriter out) {
		renderStartTagWithAttributes(out);
		label.render(out);
		renderEndTag(out);
	}

}
