/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import org.databene.commons.ArrayFormat;
import org.databene.commons.ArrayUtil;
import org.databene.commons.StringUtil;
import org.databene.jdbacl.SQLUtil;
import org.databene.jdbacl.model.DBTable;

/**
 * Abstract implementation of the {@link CheckStrategy} interface.<br/><br/>
 * Created: 05.11.2011 07:52:17
 * @since 0.9.1
 * @author Volker Bergmann
 */
public abstract class AbstractCheckStrategy implements CheckStrategy {
	
	protected final String defectType;
	
	public AbstractCheckStrategy(String defectType) {
		this.defectType = defectType;
	}
	
	public String getDefectType() {
		return defectType;
	}
	
	protected String renderReportColumns(String reportColumnsSpec, DBTable table, String tableAlias) {
		return ArrayFormat.format(parseReportColumns(reportColumnsSpec, table, tableAlias));
	}

	protected String[] parseReportColumns(String reportColumnsSpec, DBTable table) {
		return parseReportColumns(reportColumnsSpec, table, null);
	}

	protected String[] parseReportColumns(String reportColumnsSpec, DBTable table, String tableAlias) {
		if (!StringUtil.isEmpty(reportColumnsSpec))
			return StringUtil.trimAll((String[]) ArrayFormat.parse(reportColumnsSpec, ",", String.class));
		return SQLUtil.prependAlias(tableAlias, defaultReportColumns(table));
	}

	protected String renderReportColumns(String reportColumnsSpec, DBTable table) {
		return ArrayFormat.format(parseReportColumns(reportColumnsSpec, table));
	}

	protected String[] defaultReportColumns(DBTable table) {
		String[] pkColumnNames = table.getPKColumnNames();
		if (!ArrayUtil.isEmpty(pkColumnNames))
			return pkColumnNames;
		else
			return table.getColumnNames();
	}
	
	protected String[] appendColumnsIfNotIncluded(String candidates[], String[] columns) {
		for (String candidate : candidates)
			columns = appendColumnIfNotIncluded(candidate, columns);
		return columns;
	}
	
	protected String[] appendColumnIfNotIncluded(String candidate, String[] columns) {
		if (!StringUtil.containsIgnoreCase(candidate, columns))
			columns = ArrayUtil.append(candidate, columns);
		return columns;
	}
	
}
