package org.databene.dbsanity.model;

import java.io.File;

import org.databene.commons.Filter;

/**
 * Parent class for composite sanity checks. 
 * It provides methods to execute the sub checks and query their results.
 * @since 1.0
 * @author Yibo Wang
 * @author Volker Bergmann
 */
public abstract class AbstractSanityCheckSuite implements SanityCheckSuite {
	
	protected SuiteHolder parent;
	protected String name;
	protected String ignorableTables;
	protected File reportFolder;

	public AbstractSanityCheckSuite(String name, File reportFolder, SuiteHolder parent) {
	    this.name = name;
	    this.reportFolder = reportFolder;
		this.parent = parent;
    }

	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public SuiteHolder getParent() {
		return parent;
	}

	public SanityCheckSuite getRootSuite() {
		SanityCheckSuite result = this;
		while (result.getParent() != null)
			result = result.getParent();
		return result;
	}

	public void setIgnorableTables(String ignorableTables) {
		this.ignorableTables = ignorableTables;
	}
	
	public File getReportFolder() {
    	return reportFolder;
    }

	final public SanityCheckVerdict getVerdict() {
		if (countErredChecks() > 0)
			return SanityCheckVerdict.ERROR;
		else if (countFailedChecks() > 0)
			return SanityCheckVerdict.FAILED;
		else if (countSkippedChecks() > 0)
			return SanityCheckVerdict.SKIPPED;
		else if (countPassedChecks() > 0)
			return SanityCheckVerdict.PASSED;
		else if (countPendingChecks() > 0)
			return SanityCheckVerdict.PENDING;
		else
			return SanityCheckVerdict.NOT_APPLICABLE;
	}

	final public int countApplicableChecks() {
		return countChecks(new ApplicableCheckFilter());
	}

    final public int countPendingChecks() {
		return countChecksWithVerdict(SanityCheckVerdict.PENDING);
	}

    final public int countSkippedChecks() {
		return countChecksWithVerdict(SanityCheckVerdict.SKIPPED);
	}
	
	final public int countPassedChecks() {
		return countChecksWithVerdict(SanityCheckVerdict.PASSED);
	}
	
    final public int countFailedChecks() {
		return countChecksWithVerdict(SanityCheckVerdict.FAILED);
	}
	
    final public int countErredChecks() {
		return countChecksWithVerdict(SanityCheckVerdict.ERROR);
	}

	public abstract int countChecks(Filter<SanityCheck> filter);
	
	public abstract int countChecksWithVerdict(SanityCheckVerdict verdict);
}
