/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.sql.Connection;
import java.util.Locale;
import java.util.ResourceBundle;

import org.databene.jdbacl.DatabaseDialect;
import org.databene.jdbacl.DatabaseDialectManager;
import org.databene.jdbacl.identity.IdentityProvider;
import org.databene.jdbacl.identity.KeyMapper;
import org.databene.jdbacl.identity.mem.MemKeyMapper;
import org.databene.jdbacl.model.Database;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Gives access to common objects useful for checks.<br/><br/>
 * Created: 06.11.2011 17:31:28
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class CheckContext {

	private static final Logger LOGGER = LoggerFactory.getLogger(CheckContext.class);
	
	private Connection connection;
	private Database database;
	private IdentityProvider identityProvider;
	private KeyMapper keyMapper;
	private int totalCheckCount;
	private int pendingCheckCount;
	private ResourceBundle resourceBundle;
	
	public CheckContext(Connection connection, Database database, int pendingCheckCount) {
		this(connection, database, ResourceBundle.getBundle("dbsanity", Locale.getDefault()), pendingCheckCount);
	}

	public CheckContext(Connection connection, Database database, ResourceBundle resourceBundle, int pendingCheckCount) {
		this.connection = connection;
		this.database = database;
		this.identityProvider = new IdentityProvider();
		this.keyMapper = new MemKeyMapper(connection, "db", null, null, identityProvider, database);
		this.resourceBundle = resourceBundle;
		this.pendingCheckCount = pendingCheckCount;
		this.totalCheckCount = pendingCheckCount;
	}
	
	public Connection getConnection() {
		return connection;
	}
	
	public Database getDatabase() {
		return database;
	}
	
	public DatabaseDialect getDialect() {
		return DatabaseDialectManager.getDialectForProduct(
				database.getDatabaseProductName(), database.getDatabaseProductVersion());
	}
	
	public String localize(String key) {
		try {
			return resourceBundle.getString(key);
		} catch (Exception e) {
			LOGGER.error("Error retrieving key " + key, e);
			return key; 
		}
	}
	
	public IdentityProvider getIdentityProvider() {
		return identityProvider;
	}
	
	public KeyMapper getKeyMapper() {
		return keyMapper;
	}
	
	public void checkCompleted() {
		this.pendingCheckCount--;
	}
	
	public int getPendingCheckCount() {
		return pendingCheckCount;
	}
	
	public int getTotalCheckCount() {
		return totalCheckCount;
	}
	
}
