/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.ArrayList;

import org.databene.commons.collection.MarkedList;

/**
 * Represents a (usually database) row of defective data.<br/><br/>
 * Created: 25.02.2012 08:02:58
 * @since 0.9.4
 * @author Volker Bergmann
 */
public class DefectRow {
	
	private MarkedList<Object> cells;
	
	public DefectRow(int columnCount) {
		this.cells = new MarkedList<Object>(new ArrayList<Object>(columnCount));
		for (int i = 0; i < columnCount; i++)
			cells.add(null);
	}

	public DefectRow(Object[] cells) {
		this.cells = new MarkedList<Object>(new ArrayList<Object>(cells.length));
		for (int i = 0; i < cells.length; i++)
			this.cells.add(cells[i]);
	}

	public int length() {
		return cells.size();
	}

	public Object get(int index) {
		return cells.get(index);
	}

	public void set(int index, Object value) {
		set(index, value, false);
	}

	public void set(int index, Object value, boolean marked) {
		cells.set(index, value);
		if (marked)
			cells.mark(index);
	}

	public DefectRow add(Object cell) {
		return add(cell, false);
	}
	
	public DefectRow add(Object cell, boolean marked) {
		cells.add(cell);
		if (marked)
			cells.mark(cells.size() - 1);
		return this;
	}
	
	public boolean isMarked(int index) {
		return cells.isMarked(index);
	}
	
	public void mark(int index) {
		this.cells.mark(index);
	}

	public Object[] getCells() {
		return cells.toArray();
	}
	
	@Override
	public String toString() {
		return getClass().getSimpleName() + cells;
	}
	
}
