/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import org.databene.commons.StringUtil;
import org.databene.commons.SystemInfo;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.SQLUtil;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;

/**
 * Verifies that the values of a column (group) are valid foreign key references.<br/><br/>
 * Created: 05.11.2011 09:00:42
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class ForeignKeyStrategy extends DefectQueryStrategy {

	String refererTableName;
	String[] refererColumns;
	String refereeTableName;
	String refereeColumnsSpec;
	String reportColumnsSpec;
	
	public ForeignKeyStrategy(String refererTableName, String[] refererColumns, String refereeTableName, String refereeColumnsSpec, String reportColumnsSpec) {
		super(null, "foreignKey");
		this.refererTableName = refererTableName;
		this.refererColumns = refererColumns;
		this.refereeTableName = refereeTableName;
		this.refereeColumnsSpec = refereeColumnsSpec;
		this.reportColumnsSpec = reportColumnsSpec;
	}

	@Override
	public DefectIterator perform(CheckContext context) {
		Database database = context.getDatabase();
		DBTable refererTable = database.getTable(refererTableName);
		DBTable refereeTable = database.getTable(refereeTableName);
		String[] refereeColumns;
		if (refereeColumnsSpec != null) {
			refereeColumns = refereeColumnsSpec.split(",");
		} else {
			// if refereeColumns spec is missing then assume the referee's primary key columns
			refereeColumns = refereeTable.getPKColumnNames();
		}
		StringUtil.trimAll(refereeColumns);
		
		// create query
	    String LF = SystemInfo.getLineSeparator();
		String reportColumns = renderReportColumns(reportColumnsSpec, refererTable, "referer");
		this.query = "select " + reportColumns + " from " + refererTable.getName() + " referer " + LF + 
			'\t' + SQLUtil.leftJoin("referer", refererColumns, refereeTableName, "referee", refereeColumns) + LF + 
			"where referer." + refererColumns[0] + " is not null " + LF +
			"\tand referee." + refereeColumns[0] + " is null";
		return super.perform(context);
	}
	
}
