/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.io.File;
import java.util.Date;

import org.databene.commons.Filter;
import org.databene.dbsanity.Reporter;

/**
 * Parent interface of all check suite classes.<br/><br/>
 * Created: 12.11.2011 11:11:03
 * @since 0.9.2
 * @author Volker Bergmann
 */
public interface SanityCheckSuite extends Reportable {

	SuiteHolder getParent();
	SanityCheckSuite getRootSuite();
	File getReportFolder();
	void setIgnorableTables(String ignorableTables);
	int countChecks();

	void perform(CheckContext checkContext, Reporter reporter);
	void reset();

	Date getStartDate();
    int countApplicableChecks();
    int countPendingChecks();
	int countSkippedChecks();
    int countPassedChecks();
    int countFailedChecks();
    int countErredChecks();
    
	int countChecks(Filter<SanityCheck> filter);
	int countChecksWithVerdict(SanityCheckVerdict verdict);
	
}
