/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.io.File;
import java.util.List;
import java.util.Set;

import org.databene.commons.Filter;
import org.databene.commons.Visitor;
import org.databene.commons.version.VersionNumber;
import org.databene.dbsanity.Reporter;

/**
 * Default implementation of the the interfaces {@link SuiteHolder} and {@link CheckHolder}. 
 * An instance of this class can hold checks as well as sub suites.<br/><br/>
 * Created: 13.11.2011 07:02:06
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class SuiteAndCheckHolder extends SuiteHolderSupport implements CheckHolder {
	
	CheckHolderSupport checkHolderSupport;

	public SuiteAndCheckHolder(String name, File reportFolder, SuiteHolder parent) {
		super(name, reportFolder, parent);
		this.checkHolderSupport = new CheckHolderSupport(name, reportFolder, parent);
	}

	public void addCheck(SanityCheck check) {
		checkHolderSupport.addCheck(check);
	}

	public List<SanityCheck> getChecks() {
		return checkHolderSupport.getChecks();
	}

	public int countChecks() {
		return super.countChecks() + checkHolderSupport.countChecks();
	}

	public void perform(CheckContext checkContext, Reporter reporter) {
		super.perform(checkContext, reporter);
		checkHolderSupport.perform(checkContext, reporter);
	}

	public int countChecksWithVerdict(SanityCheckVerdict verdict) {
		return super.countChecksWithVerdict(verdict) + checkHolderSupport.countChecksWithVerdict(verdict);
	}

	public int countChecks(Filter<SanityCheck> filter) {
		return super.countChecks(filter) + checkHolderSupport.countChecks(filter);
	}

	public long getDefectCount() {
		return super.getDefectCount() + checkHolderSupport.getDefectCount();
	}
	
	@Override
	public int getExecutionTime() {
		return super.getExecutionTime() + checkHolderSupport.getExecutionTime();
	}

	public void accept(Visitor<Reportable> visitor) {
		super.accept(visitor);
		checkHolderSupport.accept(visitor);
	}

	public void checkApplicability(VersionNumber appVersion, Set<String> tablesToCheck, Set<String> tags) {
		super.checkApplicability(appVersion, tablesToCheck, tags);
		checkHolderSupport.checkApplicability(appVersion, tablesToCheck, tags);
	}

	public void reset() {
		super.reset();
		checkHolderSupport.reset();
	}

	public SuiteAndCheckHolder getChildSuite(String childName) {
		return (SuiteAndCheckHolder) childSuites.get(childName);
	}


}
