/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.HashMap;
import java.util.Map;

import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.DatabaseDialect;
import org.databene.jdbacl.DatabaseDialectManager;
import org.databene.jdbacl.model.DBTable;
import org.databene.jdbacl.model.Database;
import org.xml.sax.SAXException;

/**
 * {@link CheckStrategy} checking for trimming of strings (char, varchar, clob, nvarchar, nclob, ...).<br/><br/>
 * Created: 18.03.2012 18:54:29
 * @since 0.9.4
 * @author Volker Bergmann
 */
public class TrimmedStrategy extends DefectQueryStrategy {
	
	private String tableName;
	private String columnName;
	private String reportColumnsSpec;

	public TrimmedStrategy(String tableName, String column, String reportColumnsSpec) {
		super(null, "trimmed");
		this.tableName = tableName.toUpperCase();
		this.columnName = column.toUpperCase();
		this.reportColumnsSpec = reportColumnsSpec;
	}
	
	@Override
	public DefectIterator perform(CheckContext context) {
		Database database = context.getDatabase();
		DatabaseDialect dialect = DatabaseDialectManager.getDialectForProduct(
				database.getDatabaseProductName(), database.getDatabaseProductVersion());
		DBTable table = database.getTable(tableName);
		this.query = "SELECT " + renderReportColumns(reportColumnsSpec, table) + " FROM " + tableName + 
			" WHERE " + columnName + " IS NOT NULL AND " + dialect.trim(columnName) + " != " + columnName;
		return super.perform(context);
	}
	
	@Override
	public void serialize(SimpleXMLWriter writer) throws SAXException {
		Map<String, String> atts = new HashMap<String, String>();
		atts.put("column", columnName);
		writer.startElement("trimmed", atts);
		writer.endElement("trimmed");
	}
	
	@Override
	protected String[] defaultReportColumns(DBTable table) {
		return appendColumnIfNotIncluded(columnName, super.defaultReportColumns(table));
	}

	@Override
	public String toString() {
		return "trimmed[" + tableName + '.' + columnName + "]";
	}
	
}
