/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.HashMap;
import java.util.Map;

import org.databene.commons.ArrayFormat;
import org.databene.commons.SystemInfo;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.model.DBTable;
import org.xml.sax.SAXException;

/**
 * Verifies that a column or a combination of several columns has unique values.<br/><br/>
 * Created: 05.11.2011 08:11:12
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class UniqueQueryStrategy extends DefectQueryStrategy {
	
	private String tableName;
	private String[] columns;
	private String reportColumnsSpec;

	public UniqueQueryStrategy(String tableName, String[] columns, String reportColumnsSpec) {
		super(null, "unique");
		this.tableName = tableName;
		this.columns = columns;
		this.reportColumnsSpec = reportColumnsSpec;
	}

	@Override
	public DefectIterator perform(CheckContext context) {
		// create query
		DBTable table = context.getDatabase().getTable(tableName);
	    String LF = SystemInfo.getLineSeparator();
		this.query = "select " + renderReportColumns(reportColumnsSpec, table) + " from " + table + " " + LF +
			"where " + joinColumnValues(columns) + " in (" + LF +
			"	select " + joinColumnValues(columns) + " from " + table + 
			" group by " + ArrayFormat.format(columns) + " having count(*) > 1" + LF +
			")";
		return super.perform(context);
	}

	private String joinColumnValues(String[] columns) {
		StringBuilder builder = new StringBuilder();
		for (int i = 0; i < columns.length; i++) {
			if (i > 0)
				builder.append(" || '|' || ");
			builder.append(columns[i]);
		}
		return builder.toString();
	}
	
	@Override
	protected String[] defaultReportColumns(DBTable table) {
		return appendColumnsIfNotIncluded(columns, super.defaultReportColumns(table));
	}
	
	@Override
	public void serialize(SimpleXMLWriter writer) throws SAXException {
		Map<String, String> atts = new HashMap<String, String>();
		atts.put("columns", ArrayFormat.format(columns));
		atts.put("reportColumns", reportColumnsSpec);
		writer.startElement("unique", atts);
		writer.endElement("unique");
	}
	
	@Override
	public String toString() {
		return "unique[" + tableName + '(' + ArrayFormat.format(columns) + ")]";
	}
	
}
