/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.databene.commons.ArrayFormat;
import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.databene.jdbacl.model.DBTable;
import org.xml.sax.SAXException;

/**
 * Verifies that an expression provides only values from a predefined set.<br/><br/>
 * Created: 05.11.2011 08:01:20
 * @since 0.9.1
 * @author Volker Bergmann
 */
public class ValuesQueryStrategy extends DefectQueryStrategy {

	private String tableName;
	private String expression;
	private String condition;
	private List<String> values;
	private String reportColumnsSpec;
	
	public ValuesQueryStrategy(String tableName, String expression, String condition,
			List<String> values, String reportColumnsSpec) {
		super(null, "values");
		this.tableName = tableName; 
		this.expression = expression;
		this.condition = condition;
		this.values = values;
		this.reportColumnsSpec = reportColumnsSpec;
	}

	@Override
	public DefectIterator perform(CheckContext context) {
		// create query
		DBTable table = context.getDatabase().getTable(tableName);
		String[] reportColumns = parseReportColumns(reportColumnsSpec, table);
		reportColumns = appendColumnIfNotIncluded(expression, reportColumns);
		StringBuilder builder = new StringBuilder("select ").append(ArrayFormat.format(reportColumns));
		builder.append(" from ").append(tableName);
		builder.append(" where ").append(expression).append(" not in (");
		for (int i = 0; i < values.size(); i++) {
			if (i > 0)
				builder.append(", ");
			builder.append((Object) values.get(i));
		}
		builder.append(")");
		if (!StringUtil.isEmpty(condition))
			builder.append(" AND ").append(condition);
		this.query = builder.toString();
		return super.perform(context);
	}
	
	@Override
	public void serialize(SimpleXMLWriter writer) throws SAXException {
		Map<String, String> atts = new HashMap<String, String>();
		atts.put("expression", expression);
		atts.put("values", CollectionUtil.formatCommaSeparatedList(values, null));
		atts.put("condition", condition);
		atts.put("reportColumns", reportColumnsSpec);
		writer.startElement("values", atts);
		writer.endElement("values");
	}
	
	@Override
	public String toString() {
		return "values" + values;
	}
	
}
