/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.data;

import java.util.Collection;

import org.databene.commons.HeavyweightIterable;
import org.databene.dbsanity.model.CheckContext;
import org.databene.dbsanity.model.DefectRow;

/**
 * {@link DataMatcher}.<br/><br/>
 * Created: 25.01.2012 16:45:18
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class SubsetMatcher extends DataMatcher {

	public SubsetMatcher(HeavyweightIterable<Object[]> reference, int[] idColumnIndexes, CheckContext context) {
		super(reference, idColumnIndexes, context);
	}

	@Override
	public DefectRow check(TesteeSample testeeSample) {
		Object[] referenceSample = findAndMarkReferenceSampleByIdentity(testeeSample);
		if (referenceSample == null)
			return null; // reference is subset, so any additional data is allowed
		else
			return compare(referenceSample, testeeSample);
	}

	@Override
	public Collection<Object[]> getMissingSamples() {
		return getUnmarkedReferenceSamples();
	}

}
