/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.data;

import org.databene.commons.Assert;
import org.databene.commons.HeavyweightIterable;
import org.databene.commons.ProgrammerError;
import org.databene.commons.Tabular;
import org.databene.dbsanity.model.CheckContext;

/**
 * Abstract parent class for classes that import testee data from a database table.<br/><br/>
 * Created: 25.01.2012 12:15:18
 * @since 0.9.3
 * @author Volker Bergmann
 */
public abstract class Testee implements Tabular, HeavyweightIterable<TesteeSample> {
	
	private boolean initialized;
	protected CheckContext context;
	
	public Testee() {
		this.context = null;
		this.initialized = false;
	}
	
	public final void init(CheckContext context) {
		Assert.notNull(context, "connection");
		this.context = context;
		onInit();
		this.initialized = true;
	}
	
	protected void assertInitialized() {
		if (!initialized)
			throw new ProgrammerError(this + " has been invoked without initialization");
	}
	
	protected boolean isInitialized() {
		return initialized;
	}
	
	public abstract TesteeSampleIterator iterator();
	protected abstract void onInit();
	public abstract String getQuery();
	public abstract int[] getIdentityColumnIndexes();
	public abstract int[] getCheckedColumnIndexes();
	
}
