/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.data;

import org.databene.commons.ArrayFormat;
import org.databene.commons.bean.ArrayWithIdentity;
import org.databene.commons.converter.ToStringConverter;

/**
 * Represents a data row of a testee to be verified against a set of data rows from a reference data set.<br/><br/>
 * Created: 25.02.2012 11:34:35
 * @since 0.9.4
 * @author Volker Bergmann
 */
public class TesteeSample {
	
	private Object[] cells;
	private Testee testee;
	
	public TesteeSample(Object[] cells, Testee testee) {
		this.cells = cells;
		this.testee = testee;
	}
	
	public Object[] extractCheckedElements() {
		int[] checkedColumnIndexes = testee.getCheckedColumnIndexes();
		if (checkedColumnIndexes == null)
			return cells;
		Object[] result = new Object[checkedColumnIndexes.length];
		for (int i = 0; i < checkedColumnIndexes.length; i++)
			result[i] = ToStringConverter.convert(cells[checkedColumnIndexes[i]], null);
		return result;
	}

	public ArrayWithIdentity getId() {
		return IdentityUtil.identity(extractCheckedElements(), testee.getIdentityColumnIndexes());
	}

	public Object[] getCells() {
		return cells;
	}

	public int getHeadingUncheckedColumnCount() {
		return testee.getCheckedColumnIndexes()[0];
	}
	
	@Override
	public String toString() {
		return ArrayFormat.format(cells);
	}
	
}
