/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.defect;

import java.io.IOException;

import org.databene.commons.iterator.TabularIterator;
import org.databene.dbsanity.model.DefectIterator;
import org.databene.dbsanity.model.DefectRow;

/**
 * Wraps a {@link TabularIterator} with the interface of a {@link DefectIterator}.<br/><br/>
 * Created: 25.02.2012 09:29:57
 * @since 0.9.4
 * @author Volker Bergmann
 */
public class TabularDefectIteratorProxy implements DefectIterator {

	private TabularIterator source;
	
	public TabularDefectIteratorProxy(TabularIterator source) {
		this.source = source;
	}

	public String[] getColumnNames() {
		return source.getColumnNames();
	}

	public boolean hasNext() {
		return source.hasNext();
	}

	public DefectRow next() {
		return new DefectRow(source.next());
	}

	public void remove() {
		source.remove();
	}

	public void close() throws IOException {
		source.close();
	}

}
