/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.CardinalityQueryStrategy;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.w3c.dom.Element;

/**
 * Parses &lt;cardinality&gt; elements.<br/><br/>
 * Created: 28.10.2011 08:26:54
 * @since 0.9.0
 * @author Volker Bergmann
 */
public class CardinalityParser extends AbstractDbSanityXMLParser {
	
	private static final String REFERER_COLUMNS = "refererColumns";
	private static final String REFEREE_TABLE = "refereeTable";
	private static final String REFEREE_COLUMNS = "refereeColumns";
	private static final String DEFAULT_DEFECT_TYPE = "cardinality";

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(REFERER_COLUMNS);
	
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(
			REPORT_COLUMNS, REFEREE_TABLE, REFEREE_COLUMNS, MIN, MAX);

	public CardinalityParser() {
		super("cardinality", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {

		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);
		String refererTableName = getTableName(parent, XMLUtil.format(element));

		// parse XML element
		String[] refererColumns = getRequiredAttribute(REFERER_COLUMNS, element).split(",");
		StringUtil.trimAll(refererColumns);
		String refereeTableSpec = getOptionalAttribute(REFEREE_TABLE, element);
		String refereeColumnsSpec = getOptionalAttribute(REFEREE_COLUMNS, element);
		Long min = parseMin(element);
		Long max = parseMax(element);
		String reportColumnsSpec = getReportColumnsSpec(element);
		String code = XMLUtil.format(element);
		DefectQueryStrategy strategy = new CardinalityQueryStrategy(refererTableName, refererColumns, 
				refereeTableSpec, refereeColumnsSpec, reportColumnsSpec, min, max, code);
		parent.setStrategy(strategy);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return strategy;
	}

}
