/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import org.databene.commons.ConfigurationError;
import org.databene.jdbacl.identity.IdentityProvider;
import org.databene.jdbacl.identity.xml.IdentityParseContext;
import org.databene.webdecs.xml.ParseContext;

/**
 * {@link ParseContext} for DB Sanity.<br/><br/>
 * Created: 05.12.2010 12:47:26
 * @since 0.4
 * @author Volker Bergmann
 */
public class DbSanityParseContext extends IdentityParseContext {
	
	private Set<String> checks = new HashSet<String>();
	
	protected DbSanityParseContext parentContext;
	File sourceFile;
	File tempFolder;
	File reportFolder;

	public DbSanityParseContext(File reportFolder, File tempFolder) {
		this(reportFolder, tempFolder, new IdentityProvider());
	}

	public DbSanityParseContext(File reportFolder, File tempFolder, IdentityProvider identityProvider) {
		super(identityProvider);
		this.parentContext = null;
		this.sourceFile = null;
		this.reportFolder = reportFolder;
		this.tempFolder = tempFolder;
		this.identityProvider = identityProvider;
		createParsers();
	}

	private DbSanityParseContext(
			DbSanityParseContext parentContext, File sourceFile, File reportFolder, File tempFolder) {
		super(null);
		this.parentContext = parentContext;
		this.sourceFile = sourceFile;
		this.reportFolder = reportFolder;
		this.tempFolder = tempFolder;
		createParsers();
	}
	
	public DbSanityParseContext createSubContext(File sourceFile, File reportFolder, File tempFolder) {
		return new DbSanityParseContext(this, sourceFile, reportFolder, tempFolder);
	}

	private void createParsers() {
		addParser(new DbSanityParser());
		addParser(new IdentityCheckParser());
		addParser(new CheckParser());
		addParser(new DescriptionParser());
		addParser(new DefectQueryParser());
		addParser(new ForeignKeyParser());
		addParser(new UniqueParser());
		addParser(new NotNullParser());
		addParser(new NumberSizeParser());
		addParser(new StringLengthParser());
		addParser(new PatternParser());
		addParser(new VersionProviderParser());
		addParser(new ValuesParser());
		addParser(new ValidatorParser());
		addParser(new RowCountParser());
		addParser(new CardinalityParser());
		addParser(new HierarchyProviderParser());
		addParser(new DataParser());
		addParser(new PathsParser());
	}

	public File getSourceFile() {
		return sourceFile;
	}
	
	public File getReportFolder() {
		return reportFolder;
	}
	
	public File getTempFolder() {
		return tempFolder;
	}

	public IdentityProvider getIdentityProvider() {
		if (parentContext != null)
			return parentContext.getIdentityProvider();
		else
			return super.getIdentityProvider();
	}

	public void checkUniqueness(String checkName) {
		if (parentContext != null)
			parentContext.checkUniqueness(checkName);
		else {
			if (checks.contains(checkName))
				throw new ConfigurationError("Duplicate check name: " + checkName);
			checks.add(checkName);
		}
    }

}
