/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import org.databene.commons.version.Versions;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.nk.IdentityCheck;
import org.databene.jdbacl.identity.IdentityModel;
import org.databene.jdbacl.identity.xml.IdentityParser;
import org.w3c.dom.Element;

/**
 * Parses an &lt;identity&gt; element in a DB Sanity XML file.<br/><br/>
 * Created: 05.12.2010 14:39:48
 * @since 0.4
 * @author Volker Bergmann
 */
public class IdentityCheckParser extends AbstractDbSanityXMLParser { // TODO v0.x test
	
	private IdentityParser identityParser;
	
	public IdentityCheckParser() {
		super(IdentityParser.IDENTITY, IdentityParser.REQUIRED_ATTRIBUTES, IdentityParser.OPTIONAL_ATTRIBUTES, SanityCheckFile.class);
		this.identityParser = new IdentityParser();
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		IdentityModel identity = (IdentityModel) identityParser.parse(element, parentPath, context);
	    // TODO v0.x parseUnimportantColumnsOfTable(element, tableConfig, module);
		SanityCheckFile file = getParentSanityCheckFile(parentPath);
		Versions affectedVersions = Versions.valueOf(getOptionalAttribute("versions", element));
		file.addCheck(new IdentityCheck(createCheckName(identity.getTableName()), identity, affectedVersions, "db", 
				context.getSourceFile(), file.getTempFolder(), file.getReportFolder(), file.getDocPage()));
		return identity;
	}

	public static String createCheckName(String tableName) {
		return tableName + "-identity";
	}

}
