/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.dbsanity.model.PathsQueryStrategy;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.query.DefectQueryStrategy;
import org.w3c.dom.Element;

/**
 * Parses &lt;path&gt; checks.<br/><br/>
 * Created: 23.03.2012 20:29:28
 * @since 0.9.4
 * @author Volker Bergmann
 */
public class PathsParser extends AbstractDbSanityXMLParser {

	private static final String MAIN_PATH = "reference";
	private static final String ALTERNATIVE_PATH = "testee";
	private static final String EQUAL = "equal";
	
	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(MAIN_PATH, ALTERNATIVE_PATH);
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(CONDITION, REPORT_COLUMNS, EQUAL);

	public PathsParser() {
		super("paths", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);

		// parse XML element
		String referenceSpec = getRequiredAttribute("reference", element);
		String testeeSpec = getRequiredAttribute("testee", element);
		String equalSpec = getOptionalAttribute("equal", element);
		boolean equal = (equalSpec != null ? Boolean.valueOf(equalSpec.trim()) : true);
		String condition = getCondition(element);		
	    String reportColumnsSpec = getReportColumnsSpec(element);
	    
		DefectQueryStrategy strategy = new PathsQueryStrategy(
				referenceSpec, testeeSpec, equal, condition, reportColumnsSpec);
		parent.setStrategy(strategy);
	    return strategy;
	}
	
}
