/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.PatternStrategy;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.w3c.dom.Element;

/**
 * Provides pattern-based checks for strings.<br/><br/>
 * Created: 18.10.2011 18:04:43
 * @since 0.8.4
 * @author Volker Bergmann
 */
public class PatternParser extends AbstractDbSanityXMLParser {
	
	private static final String REGEX = "regex";
	private static final String LIKE = "like";

	private static final String DEFAULT_DEFECT_TYPE = "pattern";

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(EXPRESSION);
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(
			REPORT_COLUMNS, REGEX, LIKE, EXECUTOR); // TODO v1.0 support language

	public PatternParser() {
		super("pattern", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);
		String sourceCode = XMLUtil.format(element);
		String tableName = getTableName(parent, sourceCode);

		// parse XML element
		String expression = getRequiredAttribute(EXPRESSION, element);
		String condition = getCondition(element);
		String regex = getOptionalAttribute(REGEX, element);
		String like  = getOptionalAttribute(LIKE,  element);
		if (StringUtil.isEmpty(regex) && StringUtil.isEmpty(like))
			throw new SyntaxError("'regex' or 'like' must be specified in <pattern>", sourceCode);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
		Executor executor = parseExecutor(element, context);
		String reportColumnsSpec = getReportColumnsSpec(element);

		PatternStrategy strategy = new PatternStrategy(tableName, expression, condition, regex, like, executor, reportColumnsSpec, sourceCode);
		parent.setStrategy(strategy);
	    return strategy;
	}

}
