/*
 * (c) Copyright 2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.StrategyBasedCheck;
import org.databene.dbsanity.model.TrimmedStrategy;
import org.w3c.dom.Element;

/**
 * Parses &lt;trimmed&gt; check definitions.<br/><br/>
 * Created: 18.03.2012 22:10:56
 * @since 0.9.4
 * @author Volker Bergmann
 */
public class TrimmedParser extends AbstractDbSanityXMLParser {

	private static final String DEFAULT_DEFECT_TYPE = "trimmed";
	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet("column");
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(REPORT_COLUMNS);
	
	public TrimmedParser() {
		super("trimmed", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, StrategyBasedCheck.class);
	}
	
	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// get parent
		StrategyBasedCheck parent = (StrategyBasedCheck) this.parent(parentPath);

		// parse XML element
		String tableName = getTableName(parent, XMLUtil.format(element));
		String column = getRequiredAttribute("column", element);
		String reportColumnsSpec = getReportColumnsSpec(element);
		
	    TrimmedStrategy strategy = new TrimmedStrategy(tableName, column, reportColumnsSpec);
		parent.setStrategy(strategy);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return strategy;
	}

}
