/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Collections;

import org.databene.commons.BeanUtil;
import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.DbSanity;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.jdbacl.VersionProvider;
import org.databene.script.DatabeneScriptParser;
import org.w3c.dom.Element;

/**
 * Parses a &lt;versionProvider&gt; element in a DB Sanity configuration file.<br/><br/>
 * Created: 17.10.2011 14:47:25
 * @since 0.8.4
 * @author Volker Bergmann
 */
public class VersionProviderParser extends AbstractDbSanityXMLParser{

	public VersionProviderParser() {
	    super("versionProvider", Collections.<String>emptySet(), CollectionUtil.toSet("class", "spec"), 
	    		DbSanity.class, SanityCheckFile.class); 
    }

	@Override
	public VersionProvider parse(Element element, Object[] parentPath, DbSanityParseContext context) {
        String beanClass = element.getAttribute("class");
        String spec = element.getAttribute("spec");
        if (!StringUtil.isEmpty(spec))
        	return (VersionProvider) DatabeneScriptParser.parseBeanSpec(spec).evaluate(context);
        else if (beanClass != null) {
	        VersionProvider bean = (VersionProvider) BeanUtil.newInstance(beanClass);
	        for (Element childElement : XMLUtil.getChildElements(element, false, "property")) {
	        	String name = getRequiredAttribute("name", childElement);
	        	String value = getRequiredAttribute("value", childElement);
	        	BeanUtil.setPropertyValue(bean, name, value, true, true);
	        }
	        return bean;
        } else
        	throw new SyntaxError("<versionProvider> is missing an attribute 'class' or 'spec'", XMLUtil.format(element));
    }

}
