/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import org.databene.commons.IOUtil;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.document.csv.CSVUtil;
import org.databene.jdbacl.SQLUtil;

/**
 * Creates a summary report in CSV format.<br/><br/>
 * Created: 14.09.2011 15:20:36
 * @since 0.8.3
 * @author Volker Bergmann
 */
public class CSVSummaryModule extends AbstractSummaryReportModule<PrintWriter> {
	
	private static final String CSV_REPORT_FILE_NAME = "dbsanity-report-summary.csv";

	private static final char DEFAULT_SEPARATOR = ',';
	
	private char separator;
	
	public CSVSummaryModule() {
		this(DEFAULT_SEPARATOR);
	}

	public CSVSummaryModule(char separator) {
		super("csv", CSV_REPORT_FILE_NAME);
		this.separator = separator;
	}
	
	@Override
	protected PrintWriter startSummaryFile(SanityCheckSuite rootSuite) throws IOException {
		File file = getFile(rootSuite);
		PrintWriter out = null;
		out = new PrintWriter(new FileWriter(file));
		String check = context.localize("header_check");
		String table = context.localize("header_table");
		String defectType = context.localize("header_defect_type");
		String verdict = context.localize("header_verdict");
		String defectCount = context.localize("header_defect_count");
		String duration = context.localize("header_duration");
		String details = context.localize("header_details");
		String[] headers =  { check, table, defectType, verdict, defectCount, duration, details };
		String line = CSVUtil.formatHeaderWithLineFeed(separator, headers);
		out.print(line);
		return out;
	}

	@Override
	protected void endSummaryFile(PrintWriter out) {
		IOUtil.close(out);
	}

	@Override
	protected void exportCheck(SanityCheck check, PrintWriter out) {
		try {
			String details = check.getDetails();
			String normalizedQuery = (details != null ? SQLUtil.normalize(details, true) : null);
			CSVUtil.writeRow(out, separator,
				check.getName(),
				check.getTable(),
				check.getDefectType(),
				check.getVerdict().name(),
				String.valueOf(check.getDefectCount()),
				String.valueOf(check.getExecutionTime()),
				normalizedQuery
			);
		} catch (IOException e) {
			throw new RuntimeException("error exorting checks to CSV file", e);
		}
	}

}
