/*
 * (c) Copyright 2011-2012 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;

import org.databene.dbsanity.html.HtmlFileImage;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.dbsanity.util.CheckVisitor;
import org.databene.html.HTMLUtil;

/**
 * {@link ReportModule} which provides a digest view of all checks contained in the suite.<br/><br/>
 * Created: 12.06.2011 18:43:25
 * @since 0.8
 * @author Volker Bergmann
 */
public class DigestModule extends AbstractReportModule {

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return 1;
	}
	
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
	    out.startModule(localize("digest"));
	    out.startPanel("style", "line-height:8px");
	    out.print("		");
	    renderIcons(suite, out);
	    out.println();
	    out.endPanel();
		out.endModule();
    }

	private void renderIcons(SanityCheckSuite suite, HtmlPrintWriter out) {
		suite.accept(new IconRenderer(out));
	}
	
	
	
	public class IconRenderer extends CheckVisitor {
		
		private HtmlPrintWriter out;

		public IconRenderer(HtmlPrintWriter out) {
			this.out = out;
		}

		public void visitCheck(SanityCheck check) {
			if (check.getVerdict() != SanityCheckVerdict.NOT_APPLICABLE)
				renderIcon(check, out);
		}

		private void renderIcon(SanityCheck check, HtmlPrintWriter out) {
			SanityCheckVerdict verdict = check.getVerdict();
			String title = HTMLUtil.escape(check.getName());
			HtmlFileImage icon = new HtmlFileImage(iconFile(verdict)).withTitle(title);
			out.render(ReportUtil.docOrDefectFileRef(check, icon));
		}

		private File iconFile(SanityCheckVerdict verdict) {
			String fileName;
			switch (verdict) {
				case ERROR   : fileName = "black.gif"; break;
				case FAILED  : fileName = "red.gif";   break;
				case PENDING : fileName = "blue.gif";  break;
				case PASSED  : fileName = "green.gif"; break;
				default 	 : fileName = "gray.gif";  break;
			}
			return new File(out.getFile().getParent(), fileName);
		}

	}

}
