/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.util.Map;
import java.util.Map.Entry;

import org.databene.dbsanity.html.Renderable;
import org.databene.dbsanity.html.RenderableText;

/**
 * {@link Entry}-based implementation of a {@link HistogramModel}.<br/><br/>
 * Created: 02.12.2011 21:26:29
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class MapEntryHistogramModel extends HistogramModel<Map.Entry<String, Long>> {

	public MapEntryHistogramModel(String name, ReportContext context) {
		super(name, "defect_count", false, "#" + ReportContext.DEFECT_COLOR, 20, false, context);
	}

	@Override
	public Renderable getLabel(Entry<String, Long> element) {
		return new RenderableText(element.getKey());
	}

	@Override
	public long getValue(Entry<String, Long> element) {
		return element.getValue();
	}

	@Override
	public Renderable formatValue(long value) {
		return new RenderableText(context.formatLong(value));
	}

}
