/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.util.ArrayList;
import java.util.List;

import org.databene.dbsanity.html.HtmlFileReference;
import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.model.SanityCheckSuite;

/**
 * Assembles all {@link AbstractSummaryReportModule}s in a wrapper module that lists all report links 
 * on the main dashboard page.<br/><br/>
 * Created: 23.01.2012 10:35:20
 * @since 0.9.3
 * @author Volker Bergmann
 */
public class SummaryReportsModule extends AbstractReportModule {
	
	List<AbstractSummaryReportModule<?>> summaryModules;
	
	public SummaryReportsModule() {
		this.summaryModules = new ArrayList<AbstractSummaryReportModule<?>>();
		this.summaryModules.add(new CSVSummaryModule());
		this.summaryModules.add(new XLSSummaryModule());
	}
	
	@Override
	public void setContext(ReportContext context) {
		super.setContext(context);
		for (AbstractSummaryReportModule<?> module : summaryModules)
			module.setContext(context);
	}
	
	// ReportModule interface implementation ---------------------------------------------------------------------------
	
	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 1 : 0);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		for (AbstractSummaryReportModule<?> module : summaryModules)
			module.summary(rootSuite);
	}

	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, HtmlPrintWriter out) {
		out.startModule(localize("report_summaries"));
		for (AbstractSummaryReportModule<?> module : summaryModules) {
			out.render(new HtmlFileReference(module.getLinkLabel(), module.getFile(suite.getRootSuite())));
			out.printLineBreak();
		}
		out.endModule();
	}
	
}
