/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import org.databene.dbsanity.html.HtmlPrintWriter;
import org.databene.dbsanity.model.SanityCheckSuite;
import static org.databene.dbsanity.report.ReportUtil.*;

/**
 * Creates a summary HTML page for a folder with sanity check files and sub folders.<br/>
 * <br/>
 * Created: 26.10.2010 12:20:25
 * @since 1.0
 * @author Volker Bergmann
 */
public abstract class VerdictSummaryModule extends AbstractReportModule {

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return 1;
	}
	
	public void renderStatsTable(SanityCheckSuite suite, HtmlPrintWriter out) {
	    out.startTable("cellpadding", "3", "cellspacing", "0");
	    String no = localize("no");
	    long defectCount = suite.getDefectCount();
		out.render(row().withClass("defect").withStyle("font-size:20").withCells(
	    	cell().rightAligned().withContent(bold(context.formatLong(defectCount, no))),
	    	cell().leftAligned().withContent(bold(localize(defectCount > 0 ? "defects" : "defect"))),
	    	cell().rightAligned().withContent(nonBreakableSpace())
	    ));
	    out.renderRow(cell().withColspan(3).withContent(nonBreakableSpace()));
	    
	    // total check count
	    out.renderRow(cell().withColspan(3).withContent(bold(suite.countApplicableChecks() + " " + localize("applicable_checks") + ":")));
	    
	    double[] percentages = { getErrorPercentage(suite), getFailurePercentage(suite), getSuccessPercentage(suite), 
	    		getSkipPercentage(suite) };
		normalizePercentages(percentages);
	    
		renderTypeCount(suite.countErredChecks(),   "faulty",   percentages[0], "color:#" + ReportContext.RED, out);
		renderTypeCount(suite.countFailedChecks(),  "violated", percentages[1], "color:#" + ReportContext.BLACK, out);
		renderTypeCount(suite.countPassedChecks(),  "passed",   percentages[2], "color:#" + ReportContext.GREEN, out);
		renderTypeCount(suite.countSkippedChecks(), "skipped",  percentages[3], "color:#" + ReportContext.BLACK, out);
		out.endTable();
    }

	private void renderTypeCount(int count, String labelKey, double percentage, String color, HtmlPrintWriter out) {
		if (count > 0)
			out.render(
				row().withStyle(color).withCells(
					cell().rightAligned().withContent(context.formatLong(count, "No")),
					cell().leftAligned().withContent(localize(labelKey)),
					cell().rightAligned().withContent("(" + context.formatDecimal(percentage) + "%)")
				)
			);
	}

	private void normalizePercentages(double[] percentages) {
		int lastNonZeroIndex = percentages.length - 1;
		while (lastNonZeroIndex > 0 && percentages[lastNonZeroIndex] == 0.)
			lastNonZeroIndex--;
		if (lastNonZeroIndex > 0) {
			double sumBefore = 0.;
			for (int i = 0; i < lastNonZeroIndex; i++)
				sumBefore += percentages[i];
			percentages[lastNonZeroIndex] = 100. - sumBefore;
		}
	}

}
