/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.util;

import java.util.ArrayList;
import java.util.List;

import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;

/**
 * Visitor implementation which aggregates {@link SanityCheck}s from a {@link SanityCheckSuite} hierarchy.
 * It can be configured to restrict collection to applicable checks.<br/><br/>
 * Created: 11.11.2011 20:07:05
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class CheckAggregator extends CheckVisitor {
	
	private boolean onlyApplicables;
	private List<SanityCheck> checks = new ArrayList<SanityCheck>();

	public CheckAggregator(boolean onlyApplicables) {
		this.onlyApplicables = onlyApplicables;
	}

	public void visitCheck(SanityCheck check) {
		if (!onlyApplicables || check.getVerdict() != SanityCheckVerdict.NOT_APPLICABLE)
			checks.add(check);
	}

	public List<SanityCheck> getChecks() {
		return checks;
	}
	
}
