/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.util;

import java.util.Map;
import java.util.Set;

import org.databene.commons.collection.MapEntry;
import org.databene.commons.collection.OrderedNameMap;
import org.databene.dbsanity.model.SanityCheck;

/**
 * Abstract {@link CheckVisitor} implementation which provides the functionality of aggregating 
 * weights based on check criterions and leaves the decisions which criterion(s) and weights to 
 * apply for a check to a child classes' implementation of the {@link #criteria(SanityCheck)} and 
 * {@link #weight(SanityCheck)} methods.<br/><br/>
 * Created: 11.11.2011 17:31:56
 * @since 0.9.2
 * @author Volker Bergmann
 */
public abstract class CheckCriteriaAggregator extends CheckVisitor {
	
	OrderedNameMap<Long> weights = OrderedNameMap.<Long>createCaseInsensitiveMap();

	public Map<String, Long> getWeights() {
		return weights;
	}

	protected void visitCheck(SanityCheck check) {
		long weight = weight(check);
		if (weight <= 0)
			return;
		Set<String> criteria = criteria(check);
		for (String criterion : criteria) {
			Map.Entry<String, Long> criterionWeight = weights.getEntry(criterion);
			if (criterionWeight == null)
				criterionWeight = new MapEntry<String, Long>(criterion, weight);
			else
				criterionWeight.setValue(criterionWeight.getValue() + weight);
			weights.put(criterionWeight.getKey(), criterionWeight.getValue());
		}
	}

	protected abstract Set<String> criteria(SanityCheck check);

	protected abstract long weight(SanityCheck check);

}
