/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.util;

import java.util.HashSet;
import java.util.Set;

import org.databene.commons.StringUtil;
import org.databene.dbsanity.model.SanityCheck;

/**
 * Abstract {@link CheckCriteriaAggregator} implementation which takes a {@link SanityCheck}'s tags and (optionally) 
 * its table name and/or author name as aggregation criterions.<br/><br/>
 * Created: 12.11.2011 09:41:47
 * @since 0.9.2
 * @author Volker Bergmann
 */
public abstract class CheckTagAggregator extends CheckCriteriaAggregator {
	
	private boolean usingNamespaces;
	private boolean usingTableNameAsTag;
	private boolean usingAuthorAsTag;
	
	public CheckTagAggregator(boolean usingNamespaces, boolean usingAuthorAsTag, boolean usingTableNamesAsTags) {
		this.usingNamespaces = usingNamespaces;
		this.usingTableNameAsTag = usingTableNamesAsTags;
		this.usingAuthorAsTag = usingAuthorAsTag;
	}

	@Override
	protected Set<String> criteria(SanityCheck check) {
		HashSet<String> tags = new HashSet<String>();
		for (String tag : check.getTags()) {
			String[] prefixAndPath = StringUtil.splitOnFirstSeparator(tag, ':');
			String namespace = prefixAndPath[0];
			String name = prefixAndPath[1];
			if (tag.contains("/")) {
				String[] pathComponents = name.split("/");
				for (String pathComponent : pathComponents) {
					tags.add((usingNamespaces && namespace != null ? namespace + ':' : "") + pathComponent);
				}
			} else
				tags.add(usingNamespaces ? tag : name);
		}
		if (usingTableNameAsTag && check.getTable() != null)
			tags.add((usingNamespaces ? "t:" : "") + check.getTable());
		if (usingAuthorAsTag && check.getAuthor() != null)
			tags.add((usingNamespaces ? "a:" : "") + check.getAuthor());
		return tags;
	}
	
}
