/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.util;

import java.io.File;

import org.databene.dbsanity.HierarchyProvider;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SuiteAndCheckHolder;

/**
 * Uses a {@link HierarchyProvider} to create a new hierarchical {@link SanityCheckSuite} structure 
 * from an arbitrary given one.<br/><br/>
 * Created: 13.11.2011 09:00:48
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class HierarchyBuilder {

	public static SanityCheckSuite buildHierarchy(HierarchyProvider hierarchyProvider, SanityCheckSuite rootSuite) {
		if (hierarchyProvider == null)
			return rootSuite;
		HierarchyVisitor visitor = new HierarchyVisitor(hierarchyProvider, 
				rootSuite.getName(), rootSuite.getReportFolder());
		rootSuite.accept(visitor);
		return visitor.getHierarchy();
	}

	static class HierarchyVisitor extends CheckVisitor {

		private SuiteAndCheckHolder hierarchyRoot;
		private HierarchyProvider hierarchyProvider;
		
		public HierarchyVisitor(HierarchyProvider hierarchyProvider, String rootName, File reportFolder) {
			this.hierarchyRoot = new SuiteAndCheckHolder(rootName, reportFolder, null);
			this.hierarchyProvider = hierarchyProvider;
		}
		
		@Override
		protected void visitCheck(SanityCheck check) {
			String[] parentPath = hierarchyProvider.getParentPath(check);
			SuiteAndCheckHolder node = hierarchyRoot;
			for (String pathComponent : parentPath)
				node = getOrCreateSubNode(node, pathComponent);
			node.addCheck(check);
		}

		private SuiteAndCheckHolder getOrCreateSubNode(SuiteAndCheckHolder node, String childName) {
			SuiteAndCheckHolder subNode = node.getChildSuite(childName);
			if (subNode == null) {
				File subReportFolder = new File(node.getReportFolder(), childName);
				subNode = new SuiteAndCheckHolder(childName, subReportFolder, node);
				node.addChildSuite(subNode);
			}
			return subNode;
		}

		public SuiteAndCheckHolder getHierarchy() {
			return hierarchyRoot;
		}

	}

}
